@@ -3,17 +3,20 @@
 #'
 #' @param speciesNames A vector of species' names to collect from GBIF.
 #' @param date Vector of length two denoting the date range to select species from. Defaults to \code{NULL}.
+#' @param gbifOpts A named list of additional options to filter the GBIF records. See \code{?rgbif::occ_search} for the filters which may be applied to the GBIF records. Defaults to \code{list(coordinateUncertaintyInMeters = '0,1000')} which specifies a desired range of coordinates uncertainty (in meters) between 0 and 1000.
 #' @param structuredData Additional datasets to integrate with the presence only GBIF data. See the \code{structured_data} function. Defaults to \code{NULL}.
 #' @param spatialCovariates Spatial covariates to include in the model. May be a \code{Raster} or \code{Spatial} object. Cannot be non-\code{NULL} if \code{worldclimCovariates} is non-\code{NULL}.
 #' @param worldclimCovariates Names of the covariates to extract from Worldclim. Defaults to \code{NULL}; cannot be non-\code{NULL} if \code{spatialCovariates} is non-\code{NULL}.
-#' @param res Resolution for the world clim covariates. Valid values are: \code{0.5,2.5,5,10}. Defaults to \code{0.5}.
+#' @param res Resolution for the WorldClim covariates. Valid values are: \code{0.5,2.5,5,10}. Defaults to \code{0.5}.
 #' @param scale Should the spatial covariates be scaled. Defaults to \code{FALSE}.
 #' @param location Which area of Norway to model. Defaults to \code{'Norway'} which suggests a model for the entire county.
 #' @param boundary SpatialPolygons object of the study area. If \code{NULL} an object may be formed with \code{location}.
 #' @param return Object to return. Has to be one of \code{c('boundary', 'species', 'species plot', 'mesh', 'mesh plot', 'model', 'predictions', 'predictions map')}.
 #' @param mesh An inla.mesh object to include in the model. Defaults to \code{NULL}.
 #' @param meshParameters A list of inla.mesh arguments to create a mesh if \code{mesh = NULL}.
-#' @param spdeModel inla.spde model used in the model. May be a named list where the name of the spde object is the name of the associated dataset. Default NULL uses inla.spde2.matern.
+#' @param spdeModel \code{inla.spde} model used in the model. May be a named list where the name of the spde object is the name of the associated dataset. Default \code{NULL} uses \code{inla.spde2.matern}.
+#' @param biasField Should a second (bias) random field be added to the GBIF data. Defaults to \code{FALSE}.
+#' @param biasModel \code{inla.spde} model used for the bias field. Requires \code{biasModel} to be \code{TRUE}. Default \code{NULL} uses \code{inla.spde2.matern}.
 #' @param projection CRS projection to use. Defaults to \code{CRS('+proj=utm +zone=32 +ellps=WGS84 +datum=WGS84 +units=m +no_defs')}.
 #' @param limit Set the number of species downloaded. Defaults to \code{10000}.
 #' @param options A list of \code{INLA} and \code{inlabru} options. Defaults to \code{NULL}.
@@ -29,10 +32,10 @@
 #' @importFrom raster crop
 #' @importFrom raster merge
 #' @importFrom raster mask
-#' @importFrom maps map
+#@importFrom maps map
 #' @importFrom spocc occ
 #' @importFrom dplyr bind_rows
-#' @importFrom maptools map2SpatialPolygons
+#@importFrom maptools map2SpatialPolygons
 #'
 #' @examples {
 #'
@@ -67,15 +70,15 @@
 
 species_model <- function(speciesNames,
                           date = NULL,
+                          gbifOpts = list(coordinateUncertaintyInMeters = '0,1000'),
                           structuredData = NULL,
                           spatialCovariates = NULL,
                           worldclimCovariates = NULL,
-                          res = 0.5,
-                          scale = FALSE,
+                          res = 0.5, scale = FALSE,
                           location = 'Norway', boundary = NULL,
-                          return = 'predictions map',
-                          mesh = NULL, meshParameters = NULL,
-                          spdeModel = NULL,
+                          return = 'predictions map', mesh = NULL,
+                          meshParameters = NULL, spdeModel = NULL,
+                          biasField = FALSE, biasModel = NULL,
                           projection = CRS('+proj=utm +zone=32 +ellps=WGS84 +datum=WGS84 +units=m +no_defs'),
                           limit = 10000, options = list(), ...) {
 
@@ -100,7 +103,7 @@ species_model <- function(speciesNames,
 
   if (length(return) > 1) stop('return must contain only one element.')
 
-  if (!return%in%c('boundary','species', 'species plot', 'mesh', 'mesh plot', 'model', 'predictions', 'predictions map')) stop('return is not one of: species plot, mesh plot, model, predictions, predictions map.')
+  if (!return%in%c('boundary','species', 'species plot', 'mesh', 'mesh plot', 'model', 'predictions', 'predictions map')) stop('return is not one of: boundary, species, species plot, mesh, mesh plot, model, predictions, predictions map.')
 
   #if (!is.null(boundary)) {
 
@@ -126,13 +129,19 @@ species_model <- function(speciesNames,
     ## probably better but super slow
     #boundary <- fhimaps::norway_nuts3_map_b2020_default_sf
     #boundary <- as(boundary,'Spatial')
-     if (location == 'Norway') {
+     if (location %in% c('Norway', 'norway')) {
 
-       norwayfill <- maps::map("world", "norway", fill=TRUE, plot=FALSE,
-                         ylim=c(58,72), xlim=c(4,32))
-       IDs <- sapply(strsplit(norwayfill$names, ":"), function(x) x[1])
-       boundary <- maptools::map2SpatialPolygons(norwayfill, IDs = IDs,
-                                          proj4string = projection)
+        if (file.exists('~/Data-raw')) norwayfill <- geodata::gadm(country = 'Norway', path = '~/Data-raw', resolution = 2)
+        else norwayfill <- geodata::gadm(country = 'Norway', path = getwd(), resolution = 2)
+
+        boundary <- as(norwayfill, 'Spatial')
+        proj4string(boundary) <- projection
+
+       #norwayfill <- maps::map("world", "norway", fill=TRUE, plot=FALSE,
+       #                   ylim=c(58,72), xlim=c(4,32))
+       #IDs <- sapply(strsplit(norwayfill$names, ":"), function(x) x[1])
+       #boundary <- maptools::map2SpatialPolygons(norwayfill, IDs = IDs,
+       #                                   proj4string = projection)
 
     }
     else {
@@ -217,8 +226,8 @@ species_model <- function(speciesNames,
                            cutoff = meshParameters$cutoff,
                            max.edge = meshParameters$max.edge,
                            offset = meshParameters$offset)
-    mesh$proj4string <- proj
-    mesh$crs <- proj
+    mesh$proj4string <- projection
+    mesh$crs <- projection
 
     if (return == 'mesh') {
 
@@ -248,17 +257,19 @@ species_model <- function(speciesNames,
   if (!missing(speciesNames)){
 
   message('Obtaining GBIF species data:')
-
+##Mighht be worth doing this as a R6 + adding argument related to occuranceStatus and individualCount (note NA)
   if (is.null(date)) {
 
     species_data <- spocc::occ(query = speciesNames,
                                limit = limit,
+                               gbifopts = gbifOpts,
                                geometry = boundary@bbox,
                                has_coords = TRUE)
 
   }
   else  species_data <- spocc::occ(query = speciesNames,
                              limit = limit,
+                             gbifopts = gbifOpts,
                              geometry = boundary@bbox,
                              has_coords = TRUE,
                              date = date)
@@ -348,9 +359,14 @@ species_model <- function(speciesNames,
                                         trialsPA = trialsPA, speciesName = 'species',
                                         Projection = projection, ...)
 
+  if (!is.null(spdeModel)) {
 
-  if (!is.null(spdeModel) && organized_data$.__enclos_env__$private$Spatial) organized_data$spatialFields$sharedField <- spdeModel
+    if (!is.null(organized_data$.__enclos_env__$private$Spatial[['sharedField']])) organized_data$spatialFields$sharedField[['sharedField']] <- spdeModel
+    else stop('spdeModel provided but no shared spatial effect included in the model.')
+  }
 
+  if (biasField) organized_model$addBias(datasetName = dataGBIF, biasField = biasModel)
+  ##Print model components here? new argument called verbose or something?
 
   message('Running model:')
 
@@ -358,7 +374,7 @@ species_model <- function(speciesNames,
   #                                    sharedspatial = TRUE, specieseffects = TRUE, spdemodel = spdeModel,
   #                                    options = options)
 
-  spatialModel <- PointedSDMs::runModel(organized_data, options = options)
+  spatialModel <- PointedSDMs::fitISDM(organized_data, options = options)
 
   if (return == 'model') {
 
